const path = require('path');
const write = require('write');

/**
 * @class WebpackJsonFileList
 * @classdesc Webpack 5 plugin which generates a JSON file with the list of chosen built Javascript assets.
 */
class WebpackJsonFileList {
    /**
     * Constructor
     *
     * @param {ConstructorOptions} options
     * @constructor
     */
    constructor(options) {
        this.pluginOptions = options;
        this.assetDomain = options.domain;

        if (this.pluginOptions.assetDir.substr(-1) !== path.sep) {
            this.pluginOptions.assetDir += path.sep;
        }
    }

    apply(compiler) {
        const pluginOptions = this.pluginOptions;
        const assetDomain = this.assetDomain;

        compiler.hooks.afterEmit.tapAsync(
            'WebpackJsonFileList',
            (compilation, callback) => {
                let fileList = [];

                pluginOptions.chunks.forEach((chunk) => {
                    for (const asset in compilation.assets) {
                        const elements = asset.split('.');
                        const elementCount = elements.length;

                        const extension = elements[elementCount - 1];
                        const name = elements[0];

                        if (name === chunk && extension === "js") {
                            fileList.push(`${assetDomain}${path.sep}${pluginOptions.assetDir}${asset}`);
                        }
                    }
                });

                const output = {
                    path: `.${path.sep}${pluginOptions.outputFile}`,
                    content: JSON.stringify(fileList, null, 2)
                };

                write.sync(output.path, output.content);
                callback();
            }
        );
    }
}

module.exports = WebpackJsonFileList;

/**
 * @typedef {object} ConstructorOptions
 * @property {string} [domain] - The domain the assets can be found at
 * @property {array} chunks - A list of files (excluding extension) to include in the json-file.
 * @property {string} assetDir - The directory the asset-files are found in.
 * @property {string} outputFile - The filename for the resulting json-file.
 */
